/**************************************************************************

 File : OGF22DemoClient.java

 Authors: Valerio Venturi <valerio.venturi@cnaf.infn.it>
 Krzysztof Benedyczak <golbi@mat.umk.pl>
 
 *****************************************************************************/

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.Calendar;
import java.util.Properties;
import java.util.UUID;

import org.apache.commons.httpclient.contrib.ssl.EasySSLProtocolSocketFactory;
import org.apache.commons.httpclient.protocol.Protocol;
import org.apache.commons.httpclient.protocol.ProtocolSocketFactory;
import org.apache.commons.ssl.HttpSecureProtocol;
import org.apache.commons.ssl.KeyMaterial;
import org.apache.log4j.BasicConfigurator;

import x0Assertion.oasisNamesTcSAML2.NameIDType;
import x0Assertion.oasisNamesTcSAML2.SubjectType;
import x0Protocol.oasisNamesTcSAML2.AttributeQueryDocument;
import x0Protocol.oasisNamesTcSAML2.AttributeQueryType;
import x0Protocol.oasisNamesTcSAML2.ResponseDocument;

/**
 * @author Valerio Venturi <valerio.venturi@cnaf.infn.it>
 * @author Krzysztof Benedyczak <golbi@mat.umk.pl>
 */
public class OGF22DemoClient
{
	public static final String SAML_DNFORMAT = 
		"urn:oasis:names:tc:SAML:1.1:nameid-format:X509SubjectName";
	
	private static void doQuery(String queriedDN, String issuerName, 
			String issuerFormat, String serviceUrl)
	{
		/* create the attribute query */
    
		AttributeQueryDocument attributeQueryDocument = 
			AttributeQueryDocument.Factory.newInstance();

		AttributeQueryType attributeQuery = attributeQueryDocument
				.addNewAttributeQuery();
		attributeQuery.setID(UUID.randomUUID().toString());
		attributeQuery.setVersion("2.0");
		attributeQuery.setIssueInstant(Calendar.getInstance());

		NameIDType issuer = attributeQuery.addNewIssuer();
		issuer.setStringValue(issuerName);
		if (issuerFormat != null)
			issuer.setFormat(issuerFormat);

		SubjectType subject = attributeQuery.addNewSubject();
		NameIDType subjectNameID = subject.addNewNameID();
		subjectNameID.setFormat(SAML_DNFORMAT);
		subjectNameID.setStringValue(queriedDN);
		subject.setNameID(subjectNameID);

		// print the attribute query at stdout
		System.out.println("REQUEST:\n" + attributeQueryDocument.toString() +
				"\n");

		/* call the service */

		AttributeAuthorityServiceClient service = 
			new AttributeAuthorityServiceClient();
		AttributeAuthorityPortType port = service
				.getAttributeAuthorityPort(serviceUrl);

		ResponseDocument responseDocument = port
				.method(attributeQueryDocument);

		// print the response at sdout
		System.out.println("RESPONSE:\n" + responseDocument.toString() + 
				"\n");
	}
	
	private static void mainFunc() throws Exception
	{
		// configure logging
		BasicConfigurator.configure();

		/* get properties from etc/run.properties */

		Properties configuration = new Properties();
		configuration.load(new FileInputStream("etc/run.properties"));

		String serviceUrl = configuration.getProperty("serviceUrl");
		if (serviceUrl == null)
			throw new Error("serviceUrl not set " + "in etc/run.properties");

		String certificateFile = configuration.getProperty("certificateFile");
		if (certificateFile == null)
			throw new Error("certificateFile not set " + "in etc/run.properties");

		String privateKeyFile = configuration.getProperty("privateKeyFile");
		if (privateKeyFile == null)
			throw new Error("privateKeyFile not set in " + "etc/run.properties");

		String privateKeyPass = configuration.getProperty("privateKeyPass");
		if (privateKeyPass == null)
			throw new Error("privateKeyPass not set in " + "etc/run.properties");

    String doSelfQuery = configuration.getProperty("doSelfQuery");
    Boolean doSelfQueryB = (doSelfQuery == null ? false : Boolean.parseBoolean(doSelfQuery));
    
    /* load X.509 credentials */

		KeyMaterial key = new KeyMaterial(new File(certificateFile), 
                                       new File(privateKeyFile), 
                                       privateKeyPass.toCharArray());

		X509Certificate[] certificates = key.getAssociatedCertificateChain();
		String distinguishedName = certificates[0].getSubjectX500Principal().getName();

		/* configure HTTPS */

		HttpSecureProtocol protocolSocketFactory = new EasySSLProtocolSocketFactory();
		Protocol protocol = new Protocol("https", (ProtocolSocketFactory) protocolSocketFactory, 8443);
		protocolSocketFactory.setKeyMaterial(key);
		Protocol.registerProtocol("https", protocol);

		if (doSelfQuery == null)
		{
			System.out.println("Doing a self query for " + distinguishedName + "\n\n");
			doQuery(distinguishedName, distinguishedName,	SAML_DNFORMAT, serviceUrl);
		}
    else
		{
      InputStream certificateInputStream = new FileInputStream("etc/cert1247.pem");
      CertificateFactory certificateFactory = CertificateFactory.getInstance("X509");
      X509Certificate thirdPartySubjectCertificate = (X509Certificate) certificateFactory.generateCertificate(certificateInputStream);
      certificateInputStream.close();

      String thirdPartySubjectDistinguishedName = thirdPartySubjectCertificate.getSubjectX500Principal().getName();

			System.out.println("Quering as " + distinguishedName + " for " + thirdPartySubjectDistinguishedName + "\n\n");
			doQuery(thirdPartySubjectDistinguishedName, distinguishedName, null, serviceUrl);
		}
	}
	
	public static void main(String[] args)
	{
		try
		{
			mainFunc();
		} catch (Exception exception)
		{
			exception.printStackTrace();
		}
	}
}
